import json

from ...domain.interfaces.services.iserializer import ISerializer


class JsonSerializer(ISerializer):
    def serialize(self, data) -> str:
        return json.dumps(data)

    def deserialize(self, data: str):
        if not isinstance(data, str):
            raise TypeError("Data must be a string")
        if not data.strip():
            raise ValueError("Data cannot be empty")

        data_dict = json.loads(data)
        return data_dict

    @property
    def extension(self) -> str:
        return "json"

    @property
    def binary(self) -> bool:
        return False



from .iemail_service import *
from .ifile_manager import *
from .ipassword_manager import *
from .iserializer import *
from .istopwords_remover import *
from .itext_exporter import *
from .itranscriber import *

__all__ = [
    "ISerializer",
    "IStopwordsRemover",
    "IPasswordManager",
    "IFileManager",
    "ITextExporter",
    "ITranscriber",
    "IEmailService",
]



from .audio_service import *
from .auth_service import *
from .storage_service import *
from .user_service import *

__all__ = [
    "UserService",
    "AuthService",
    "AudioRecordService",
    "AudioTagService",
    "AudioTextService",
    "StorageService",
]



from .audio_mapper import AudioRecordMapper
from .storage_mapper import StorageMapper
from .user_mapper import UserMapper
from ...domain.interfaces import IMapper


class EntityMapperFactory(object):
    def __init__(self):
        self._mappers: dict[str, IMapper] = {
            "authuser": UserMapper(),
            "admin": UserMapper(),
            "storage": StorageMapper(),
            "audiorecord": AudioRecordMapper(),
        }

    def get_mapper(self, entity_type: str) -> IMapper:
        """
        Get serializer for an entity type.

        :param entity_type: Type of the entity (e.g., 'authuser', 'storage').
        :return: IDictable serializer.
        :raises ValueError: If serializer is not found.
        """
        serializer = self._mappers.get(entity_type)
        if serializer is None:
            raise ValueError(f"No serializer found for entity type: {entity_type}")

        return serializer

    def register_mapper(self, entity_type: str, serializer: IMapper) -> None:
        """
        Register a new serializer for an entity type.

        :param entity_type: Type of the entity.
        :param serializer: Serializer implementing IDictable.
        :raises ValueError: If entity_type is invalid.
        """
        if not entity_type:
            raise ValueError(f"Entity type is invalid: {entity_type}")
        if not isinstance(serializer, IMapper):
            raise TypeError("Serializer must implement IMapper.")

        self._mappers[entity_type] = serializer



from pydantic import BaseModel, Field


class UserDTO(BaseModel):
    entity_type: str = Field(..., pattern="^(authuser|admin)$")
    id: str
    email: str
    password_hash: str
    registration_date: str
    last_updated: str
    is_blocked: bool
    temp_password_hash: str | None = None

    class Config:
        from_attributes = True



from .audio_factory import *
from .storage_factory import *
from .user_factory import *

__all__ = [
    "AdminFactory",
    "AuthUserFactory",
    "IUserFactory",
    "AudioRecordFactory",
    "IAudioRecordFactory",
    "StorageFactory",
    "IStorageFactory",
]



from pathlib import Path

from ....domain.interfaces import ISerializer, IFileManager


class LocalFileManager(IFileManager):
    @staticmethod
    def save(data, filename: str, serializer: ISerializer) -> None:
        """Save data to file."""
        if not filename:
            raise ValueError("filename cannot be empty")

        filename = f"{filename}.{serializer.extension}"
        data = serializer.serialize(data)
        binary = serializer.binary

        file_path = Path(filename)

        with open(file_path, "wb" if binary else "w") as f:
            f.write(data)

    @staticmethod
    def load(filename: str, serializer: ISerializer):
        """Load data from file."""
        if not filename:
            raise ValueError("filename cannot be empty")

        filename = f"{filename}.{serializer.extension}"
        binary = serializer.binary

        with open(filename, "rb" if binary else "r") as f:
            data = f.read()

            return serializer.deserialize(data) if serializer else data



from abc import ABC, abstractmethod


class IPasswordManager(ABC):
    @abstractmethod
    def hash_password(self, password: str) -> str:
        """
        Hash a password.

        :param password: Password to hash.
        :return: Hash result.
        """
        pass

    @abstractmethod
    def create_password(self) -> str:
        """
        Create password.

        :return: Not hashed password."""

        pass

    @abstractmethod
    def verify_password(self, hashed_password: str, other: str) -> bool:
        pass



import msgpack

from ...domain.interfaces import ISerializer


class MsgpackSerializer(ISerializer):
    def serialize(self, data) -> bytes:
        return msgpack.packb(data, use_bin_type=True)

    def deserialize(self, data: bytes):
        if not isinstance(data, bytes):
            raise TypeError("Data must be bytes")
        if not data:
            raise ValueError("Data cannot be empty")

        return msgpack.unpackb(data, raw=False)

    @property
    def extension(self) -> str:
        return "msgpack"

    @property
    def binary(self) -> bool:
        return True



from abc import ABC, abstractmethod
from typing import Any


class ISerializer(ABC):
    @abstractmethod
    def serialize(self, data: Any) -> str | bytes: ...

    @abstractmethod
    def deserialize(self, data: str | bytes) -> Any: ...

    @property
    @abstractmethod
    def binary(self) -> bool: ...

    @property
    @abstractmethod
    def extension(self) -> str: ...



from ....domain.interfaces import (
    IUser,
    IUserRepository,
    ISerializer,
    IFileManager,
)


class LocalUserRepository(IUserRepository):
    def __init__(
        self, data_dir: str, file_manager: IFileManager, serializer: ISerializer
    ):
        """
        Create local user repository.

        :param data_dir: Directory to store local user data.
        """

        self.__serializer = serializer
        self.__file_manager = file_manager
        self._users: dict[str, IUser] = {}
        self.__dir: str = data_dir

        try:
            self._users = self.__file_manager.load(data_dir, self.__serializer)
        except Exception as e:
            pass

    def get_by_id(self, user_id: str) -> IUser | None:
        """Return user by its ID if it exists else None."""

        return self._users.get(user_id)

    def get_by_email(self, email: str) -> IUser | None:
        """Return user by email if it exists else None."""

        return next((u for u in self._users.values() if u.email == email), None)

    def add(self, user: IUser) -> None:
        """
        Add user to repository.

        :param user: New user.
        :raise ValueError: If user already exists.
        """

        if user.id in self._users:
            raise ValueError("User already exists")
        self._users[user.id] = user
        self.__save()

    def update(self, user: IUser) -> None:
        """
        Update user in repository.

        :param user: New user value.
        :raise ValueError: If user does not exist.
        """

        if user.id not in self._users:
            raise ValueError("User not found")
        self._users[user.id] = user
        self.__save()

    def delete(self, user: IUser) -> None:
        """
        Delete user from repository.

        :param user: Target user.
        :raise ValueError: If user does not exist.
        """

        if user.id not in self._users:
            raise ValueError("User not found")
        self._users.pop(user.id)
        self.__save()

    def get_all(self) -> list[IUser]:
        """Return all users in repository."""

        return list(self._users.values())

    def __save(self):
        self.__file_manager.save(self._users, self.__dir, self.__serializer)



from .audio_mapper import *
from .entity_mapper_factory import *
from .serializer_adapter import *
from .storage_mapper import *
from .user_mapper import *

__all__ = [
    "EntityMapperFactory",
    "SerializerAdapter",
]



from abc import ABC, abstractmethod


class IFileManager(ABC):
    @staticmethod
    @abstractmethod
    def save(data, path: str, serializer) -> None: ...

    @staticmethod
    @abstractmethod
    def load(path: str, serializer): ...



import logging

from password_strength import PasswordPolicy

from .storage_service import StorageService
from ...domain.factories import (
    IUserFactory,
)
from ...domain.interfaces import (
    IUserRepository,
    IUser,
)

logger = logging.getLogger(__name__)


class UserService(object):
    def __init__(
        self,
        repository: IUserRepository,
        storage_service: StorageService,
    ):
        self._repository = repository
        self._storage_service = storage_service
        self._user_factory: IUserFactory | None = None
        self._policy = PasswordPolicy.from_names(
            length=8, uppercase=1, numbers=1, special=1
        )

    def create_user(self, email: str, password: str, factory: IUserFactory) -> IUser:
        self._user_factory = factory
        user = self._user_factory.create_user(email, password)

        self._repository.add(user)
        self._storage_service.create_storage(user.id)

        return user

    def delete_user(self, user: IUser):
        self._repository.delete(user)

    def get_user_by_email(self, email: str) -> IUser | None:
        return self._repository.get_by_email(email)

    def update_user(self, user: IUser):
        self._repository.update(user)

    def get_by_id(self, user_id: str) -> IUser | None:
        return self._repository.get_by_id(user_id)

    def get_all(self) -> list[IUser]:
        return self._repository.get_all()

    def set_blocked(self, initiator: IUser, target_email: str, block: bool) -> None:
        if not initiator.can_block():
            raise PermissionError("Only admins can block users")

        user = self.get_user_by_email(target_email)
        if not user:
            raise KeyError("User not found")

        user.is_blocked = block
        self.update_user(user)

    def delete(self, initiator: IUser, target_email: str):
        if not initiator.can_block():
            raise PermissionError("Only admins can delete users")

        user = self.get_user_by_email(target_email)
        if not user:
            raise KeyError("User not found")

        self.delete_user(user)



from pydantic import BaseModel, Field


class AudioRecordDTO(BaseModel):
    entity_type: str = Field(..., pattern="^(audiorecord)$")
    id: str
    record_name: str
    file_path: str
    storage_id: str
    text: str | None = None
    language: str | None = None
    tags: list[str]
    last_updated: str

    class Config:
        from_attributes = True



from .entities.audio import AudioRecord
from .entities.storage import Storage
from .entities.user import User, AuthUser, Admin
from .exceptions import AuthException

__all__ = ["AudioRecord", "User", "AuthUser", "Admin", "AuthException", "Storage"]



from abc import ABC, abstractmethod

from ..entities.storage import Storage
from ..interfaces import IStorage


class IStorageFactory(ABC):

    @abstractmethod
    def create_storage(self, user_id: str) -> IStorage:
        pass


class StorageFactory(IStorageFactory):

    def create_storage(self, user_id: str) -> IStorage:
        return Storage(user_id)



from bson.binary import Binary
from pymongo import MongoClient
from pymongo.collection import Collection

from ....domain.interfaces import (
    IStorageRepository,
    ISerializer,
    IStorage,
)


class MongoStorageRepository(IStorageRepository):
    def __init__(
        self,
        serializer: ISerializer,
        mongo_uri: str,
        db_name: str,
        collection_name: str = "storages",
    ) -> None:

        self.__serializer = serializer

        self.__client = MongoClient(mongo_uri)
        self.__db = self.__client[db_name]
        self.__collection: Collection = self.__db[collection_name]

        self.__collection.create_index("user_id")

    def get_by_id(self, storage_id: str) -> IStorage:
        doc = self.__collection.find_one({"_id": storage_id})
        if not doc:
            raise ValueError("Storage not found")

        data = doc["data"] if self.__serializer.binary else doc["data"].decode()
        return self.__serializer.deserialize(data)

    def get_by_user(self, user_id: str) -> IStorage:
        if not user_id.strip():
            raise ValueError("user_id cannot be empty")
        doc = self.__collection.find_one({"user_id": user_id})
        if not doc:
            raise ValueError("Storage not found")

        data = doc["data"] if self.__serializer.binary else doc["data"].decode()
        return self.__serializer.deserialize(data)

    def add(self, storage: IStorage) -> None:
        if self.__collection.find_one({"_id": storage.id}):
            raise ValueError(f"Storage with ID {storage.id} already exists")
        if self.__collection.find_one({"user_id": storage.user_id}):
            raise ValueError(f"User with ID {storage.user_id} already has a storage")

        serialized = self.__serializer.serialize(storage)
        doc = {
            "_id": storage.id,
            "user_id": storage.user_id,
            "data": (
                Binary(serialized)
                if self.__serializer.binary
                else Binary(serialized.encode())
            ),
        }
        self.__collection.insert_one(doc)

    def update(self, storage: IStorage) -> None:
        if not self.__collection.find_one({"_id": storage.id}):
            raise ValueError(f"Storage with ID {storage.id} not found")

        serialized = self.__serializer.serialize(storage)
        doc = {
            "user_id": storage.user_id,
            "data": (
                Binary(serialized)
                if self.__serializer.binary
                else Binary(serialized.encode())
            ),
        }
        self.__collection.update_one({"_id": storage.id}, {"$set": doc})

    def delete(self, storage_id: str) -> None:
        doc = self.__collection.find_one({"_id": storage_id})
        if not doc:
            raise ValueError(f"Storage with ID {storage_id} not found")

        self.__collection.delete_one({"_id": storage_id})



from abc import ABC, abstractmethod

from ..entities.user import AuthUser, Admin
from ..interfaces import IUser


class IUserFactory(ABC):
    """Abstract base class for user factories."""

    @abstractmethod
    def create_user(self, email: str, password_hash: str) -> IUser:
        """Create a user with the given email and password hash."""
        pass


class AuthUserFactory(IUserFactory):
    """Factory for creating AuthUser instances."""

    def create_user(self, email: str, password_hash: str) -> IUser:
        return AuthUser(email, password_hash)


class AdminFactory(IUserFactory):
    """Factory for creating Admin instances."""

    def create_user(self, email: str, password_hash: str) -> IUser:
        return Admin(email, password_hash)



from abc import ABC, abstractmethod
from datetime import datetime


class IUser(ABC):
    @property
    @abstractmethod
    def is_blocked(self) -> bool:
        """Returns True if the user is blocked else False."""
        pass

    @is_blocked.setter
    @abstractmethod
    def is_blocked(self, is_blocked: bool):
        """Sets the blocked state of the user."""
        pass

    @property
    @abstractmethod
    def password_hash(self) -> str:
        pass

    @password_hash.setter
    @abstractmethod
    def password_hash(self, password_hash: str):
        pass

    @property
    @abstractmethod
    def temp_password_hash(self) -> str | None:
        pass

    @temp_password_hash.setter
    @abstractmethod
    def temp_password_hash(self, temp_password_hash: str):
        pass

    @property
    @abstractmethod
    def id(self) -> str:
        """Returns user ID."""
        pass

    @id.setter
    @abstractmethod
    def id(self, id_: str): ...

    @property
    @abstractmethod
    def email(self) -> str:
        """Returns user email."""
        pass

    @property
    @abstractmethod
    def registration_date(self) -> datetime:
        """Returns user registration date."""
        pass

    @property
    @abstractmethod
    def last_updated(self) -> datetime:
        """Returns user last updated date."""
        pass

    @last_updated.setter
    @abstractmethod
    def last_updated(self, last_updated: datetime): ...

    @abstractmethod
    def can_block(self) -> bool:
        pass

    @registration_date.setter
    @abstractmethod
    def registration_date(self, value):
        pass



from abc import ABC, abstractmethod

from ..entities.audio import AudioRecord
from ..interfaces import IAudioRecord


class IAudioRecordFactory(ABC):

    @abstractmethod
    def create_audio(
        self,
        file_name: str,
        file_path: str,
        storage_id: str,
        text: str,
        language: str,
    ) -> IAudioRecord:
        pass


class AudioRecordFactory(IAudioRecordFactory):

    def create_audio(
        self,
        file_name: str,
        file_path: str,
        storage_id: str,
        text: str,
        language: str,
    ) -> IAudioRecord:
        return AudioRecord(file_name, file_path, storage_id, text, language)



from .dto.storage_dto import StorageDTO
from ...domain.factories import StorageFactory, IStorageFactory
from ...domain.interfaces import IMapper, IStorage


class StorageMapper(IMapper):
    def __init__(self, factory: IStorageFactory = StorageFactory()):
        self._factory = factory

    def to_dto(self, storage: IStorage) -> StorageDTO:
        if not isinstance(storage, IStorage):
            raise TypeError("Object must be a IStorage instance")

        return StorageDTO(
            entity_type="storage",
            id=storage.id,
            user_id=storage.user_id,
            audio_record_ids=storage.audio_record_ids,
        )

    def from_dto(self, dto: StorageDTO) -> IStorage:
        storage = self._factory.create_storage(dto.user_id)
        storage.id = dto.id
        for record_id in dto.audio_record_ids:
            storage.add_audio_record(record_id)

        return storage



from uuid import uuid4

from ..interfaces import IStorage


class Storage(IStorage):
    def __init__(self, user_id: str):
        """
        Represents a user's personal storage container for audio records.

        :param user_id: The user's ID (only one user for storage).
        """

        self._id = uuid4().hex
        self._user_id = user_id
        self._audio_record_ids: list[str] = []

    @property
    def id(self) -> str:
        """Return ID of the storage container."""
        return self._id

    @id.setter
    def id(self, value: str) -> None:
        self._id = value

    @property
    def user_id(self) -> str:
        """Return the ID of owner user."""
        return self._user_id

    @property
    def audio_record_ids(self) -> list[str]:
        """Return a list of audio record IDs."""
        return self._audio_record_ids.copy()

    def add_audio_record(self, record_id: str) -> None:
        """
        Add audio record to storage.

        :param record_id: The audio record ID to add.
        :return: None
        """
        if record_id not in self._audio_record_ids:
            self._audio_record_ids.append(record_id)

    def remove_audio_record(self, record_id: str) -> None:
        """
        Remove audio record from storage.

        :param record_id: The audio record ID to remove.
        :return: None
        :raise ValueError: If record_id is not in the list of audio record IDs.
        """
        self._audio_record_ids.remove(record_id)



from abc import ABC, abstractmethod

from ...domain.interfaces import IStorage


class IStorageService(ABC):
    @abstractmethod
    def create_storage(self, user_id: str) -> IStorage: ...

    @abstractmethod
    def get_user_storage(self, user_id: str) -> IStorage | None: ...

    @abstractmethod
    def remove_audio_record(self, storage_id: str, record_id: str) -> None: ...

    @abstractmethod
    def get_all_record_ids(self, storage_id: str) -> list[str] | None: ...



from abc import ABC, abstractmethod

from ..entities.iaudio import IAudioRecord
from ..entities.istorage import IStorage
from ..entities.iuser import IUser


class IUserRepository(ABC):
    @abstractmethod
    def get_by_id(self, user_id: str) -> IUser | None: ...

    @abstractmethod
    def get_by_email(self, email: str) -> IUser | None: ...

    @abstractmethod
    def add(self, user: IUser) -> None: ...

    @abstractmethod
    def update(self, user: IUser) -> None: ...

    @abstractmethod
    def delete(self, user: IUser): ...

    @abstractmethod
    def get_all(self) -> list[IUser]: ...


class IStorageRepository(ABC):
    @abstractmethod
    def get_by_id(self, storage_id: str) -> IStorage | None: ...

    @abstractmethod
    def get_by_user(self, user_id: str) -> IStorage: ...

    @abstractmethod
    def add(self, storage: IStorage) -> None: ...

    @abstractmethod
    def update(self, storage: IStorage) -> None: ...

    @abstractmethod
    def delete(self, storage_id: str) -> None: ...


class IAudioRepository(ABC):
    @abstractmethod
    def get_by_storage(self, storage_id: str) -> tuple[IAudioRecord, ...]: ...

    @abstractmethod
    def get_by_id(self, record_id: str) -> IAudioRecord: ...

    @abstractmethod
    def add(self, record: IAudioRecord) -> None: ...

    @abstractmethod
    def update(self, record: IAudioRecord) -> None: ...

    @abstractmethod
    def delete(self, record_id: str) -> None: ...



from abc import ABC, abstractmethod


class ITranscriber(ABC):
    @abstractmethod
    def transcribe(
        self,
        content: bytes,
        language: str | None,
        max_speakers: int | None,
        main_theme: str | None,
    ) -> tuple[str, str]:
        """Transcribe audio content to text and return text and detected language."""
        pass



import logging

from audio_transcribing import NatashaStopwordsRemover as BaseStopwordsRemover

from ...domain.interfaces import IStopwordsRemover

logger = logging.getLogger(__name__)


class StopwordsRemover(IStopwordsRemover):
    def __init__(self):
        self._remover = BaseStopwordsRemover()

    def remove_stopwords(
        self, text: str, remove_swear_words: bool, go_few_times: bool
    ) -> str:
        logger.info(f"Call method remove stopwords in adapter")
        return self._remover.remove_stopwords(text, remove_swear_words, go_few_times)

    def remove_words(self, text: str, words: list[str] | tuple[str]) -> str:
        return self._remover.remove_words(text, words)



from abc import ABC, abstractmethod


class IStorage(ABC):
    @property
    @abstractmethod
    def id(self) -> str: ...

    @id.setter
    @abstractmethod
    def id(self, id_: str): ...

    @property
    @abstractmethod
    def user_id(self) -> str: ...

    @property
    @abstractmethod
    def audio_record_ids(self) -> list[str]: ...

    @abstractmethod
    def add_audio_record(self, record_id: str) -> None: ...

    @abstractmethod
    def remove_audio_record(self, record_id: str) -> None: ...



from audio_transcribing import Transcriber as BaseTranscriber

from ...domain.interfaces import ITranscriber


class Transcriber(ITranscriber):
    def __init__(
        self,
        token: str,
        whisper_model: str = "medium",
        speaker_diarization_model: str = "pyannote/speaker-diarization-3.1",
        use_faster_whisper: bool = False,
    ):
        self._transcriber = BaseTranscriber(
            token, whisper_model, speaker_diarization_model, use_faster_whisper
        )

    def transcribe(
        self,
        content: bytes,
        language: str | None,
        max_speakers: int | None,
        main_theme: str | None,
    ) -> tuple[str, str]:
        return self._transcriber.transcribe(content, language, max_speakers, main_theme)



import os

from .formats.docx_exporter import DocxExporter
from .formats.iexporter import IExporter
from ...domain.interfaces import ITextExporter


class TextExporter(ITextExporter):
    """Exporter of text (string) to different formats."""

    def __init__(self):
        self.__exporters: dict[str, IExporter] = {"docx": DocxExporter()}

    def export_text(
        self, content: str, output_dir: str, filename: str, file_format: str
    ) -> str:
        """
        Export text (string) to format.

        :param content: Target text to export.
        :param output_dir: Target output directory.
        :param filename: Filename to save the exported text.
        :param file_format: Format to save the exported text (Now only 'docx').
        :raise ValueError: If format is not supported.
        """

        if not content:
            raise ValueError("Content cannot be empty")
        if not output_dir:
            raise ValueError("Output directory cannot be empty")
        if not filename:
            raise ValueError("Filename cannot be empty")
        if not os.path.exists(output_dir):
            raise ValueError(f"Output directory does not exist: {output_dir}")
        if file_format not in self.__exporters:
            raise ValueError(f"Unsupported format: {file_format}")

        exporter = self.__exporters[file_format]
        output_path = f"{output_dir}/{filename}.{exporter.file_extension}"
        exporter.export(content, output_path)

        return output_path



from copy import copy

from ....domain.interfaces import (
    IStorageRepository,
    IStorage,
    ISerializer,
    IFileManager,
)


class LocalStorageRepository(IStorageRepository):
    def __init__(
        self, data_dir: str, file_manager: IFileManager, serializer: ISerializer
    ):
        """
        Create local storage repository.

        :param data_dir: Directory to store local storage data.
        """

        self.__serializer: ISerializer = serializer
        self.__storages: dict[str, IStorage] = {}
        self.__file_manager: IFileManager = file_manager
        self.__user_storage_map: dict[str, str] = {}
        self.__dir: str = data_dir

        try:
            self.__storages, self.__user_storage_map = self.__file_manager.load(
                data_dir,
                self.__serializer,
            )
        except:
            pass

    def get_by_id(self, storage_id: str) -> IStorage | None:
        """Return storage object copy if it exists by ID else None."""

        return copy(self.__storages.get(storage_id))

    def get_by_user(self, user_id: str) -> IStorage | None:
        """Return storage object copy if it exists by user_id else None."""

        storage_id = self.__user_storage_map.get(user_id)
        return self.__storages.get(storage_id) if storage_id else None

    def add(self, storage: IStorage) -> None:
        """
        Add storage object to storage repository.

        :param storage: Storage object.
        :raise ValueError: If storage already exists.
        """

        if storage.id in self.__storages:
            raise ValueError("Storage already exists")

        self.__storages[storage.id] = storage
        self.__user_storage_map[storage.user_id] = storage.id
        self.__save()

    def update(self, storage: IStorage) -> None:
        """
        Update storage object in storage repository.

        :param storage: New storage value.
        :raise ValueError: If storage does not exist.
        """

        if storage.id not in self.__storages:
            raise ValueError("Storage not found")
        self.__storages[storage.id] = storage
        self.__save()

    def delete(self, storage_id: str) -> None:
        if storage_id not in self.__storages:
            raise ValueError("Storage not found")

        del self.__storages[storage_id]
        self.__save()

    def __save(self):
        self.__file_manager.save(
            (self.__storages, self.__user_storage_map),
            self.__dir,
            self.__serializer,
        )



from datetime import datetime
from uuid import uuid4

from ..interfaces import IAudioRecord


class AudioRecord(IAudioRecord):
    def __init__(
        self,
        file_name: str,
        file_path: str,
        storage_id: str,
        text: str,
        language: str,
    ):
        """
        Create AudioRecord instance with basic metadata and do transcription into text with given transcribe services.

        :param file_name: Name of audio file.
        :param file_path: Full path to audio file in some storage directory (not user storage).
        :param storage_id: Storage id of audio file.
        :param text: Text of audio file.
        :param language: Language of audio file.
        """

        self._id = uuid4().hex
        self._record_name = file_name
        self._file_path = file_path
        self._storage_id = storage_id
        self._last_updated = datetime.now()
        self._text = text
        self._language = language
        self._tags = []

    @property
    def id(self) -> str:
        """Return ID of audio record."""

        return self._id

    @id.setter
    def id(self, value: str):
        """Set ID of audio record."""
        self._id = value

    @property
    def text(self) -> str:
        """Return text of audio record."""

        return self._text

    @text.setter
    def text(self, value: str) -> None:
        """Set text for audio record."""

        self._text = value
        self._last_updated = datetime.now()

    @property
    def language(self) -> str:
        """Return language of audio file."""

        return self._language

    @property
    def tags(self) -> list:
        """Return tags of audio record."""

        return self._tags.copy()

    def add_tag(self, tag_name: str, update: bool = True) -> None:
        """Add tag to audio record. Saved in lower case."""

        if tag_name.lower() not in self._tags:
            self._tags.append(tag_name.lower())
            if update:
                self._last_updated = datetime.now()

    def remove_tag(self, tag_name: str) -> None:
        """
        Remove tag from audio record.

        :raise ValueError: If tag does not exist.
        """

        self._tags.remove(tag_name.lower())
        self._last_updated = datetime.now()

    @property
    def record_name(self) -> str:
        """Return name for the audio record."""

        return self._record_name

    @record_name.setter
    def record_name(self, note_name: str):
        """Sets the display name for the audio record."""

        self._record_name = note_name
        self._last_updated = datetime.now()

    @property
    def file_path(self) -> str:
        """Absolute filesystem path to the audio file."""

        return self._file_path

    @property
    def storage_id(self) -> str:
        """Identifier of associated storage container."""

        return self._storage_id

    @property
    def last_updated(self) -> datetime:
        """Timestamp of last modification in UTC."""

        return self._last_updated

    @last_updated.setter
    def last_updated(self, value: datetime):
        self._last_updated = value



from abc import ABC, abstractmethod


class IStopwordsRemover(ABC):
    @abstractmethod
    def remove_stopwords(
        self, text: str, remove_swear_words: bool, go_few_times: bool
    ) -> str:
        """Remove stopwords from text."""
        pass

    @abstractmethod
    def remove_words(self, text: str, words: list[str] | tuple[str]) -> str:
        """Remove specific words from text."""
        pass



from pydantic import BaseModel, Field


class StorageDTO(BaseModel):
    entity_type: str = Field(..., pattern="^(storage)$")
    id: str
    user_id: str
    audio_record_ids: list[str]

    class Config:
        from_attributes = True



from bson.binary import Binary
from pymongo import MongoClient
from pymongo.collection import Collection

from ....domain.interfaces import (
    IAudioRepository,
    IStorageRepository,
    ISerializer,
    IAudioRecord,
)


class MongoAudioRepository(IAudioRepository):
    def __init__(
        self,
        storage_repository: IStorageRepository,
        serializer: ISerializer,
        mongo_uri: str,
        db_name: str,
        collection_name: str = "audio_records",
    ) -> None:

        self.__serializer = serializer
        self.__storage_repository = storage_repository

        self.__client = MongoClient(mongo_uri)
        self.__db = self.__client[db_name]

        self.__collection: Collection = self.__db[collection_name]
        self.__collection.create_index("storage_id")
        self.__collection.create_index("tags")

    def get_by_storage(self, storage_id: str) -> tuple[IAudioRecord, ...]:
        if not storage_id.strip():
            raise ValueError("Storage ID cannot be empty")

        documents = self.__collection.find({"storage_id": storage_id})
        return tuple(
            self.__serializer.deserialize(
                doc["data"] if self.__serializer.binary else doc["data"].decode()
            )
            for doc in documents
            if "data" in doc
        )

    def get_by_id(self, record_id: str) -> IAudioRecord:
        if not record_id.strip():
            raise ValueError("Record ID cannot be empty")
        doc = self.__collection.find_one({"_id": record_id})
        if not doc:
            raise ValueError("Record ID not found")

        data = doc["data"] if self.__serializer.binary else doc["data"].decode()
        return self.__serializer.deserialize(data)

    def add(self, record: IAudioRecord) -> None:
        if self.__collection.find_one({"_id": record.id}):
            raise ValueError(f"Record with ID {record.id} already exists")

        storage = self.__storage_repository.get_by_id(record.storage_id)
        if not storage:
            raise ValueError(f"Storage with ID {record.storage_id} not found")

        storage.add_audio_record(record.id)
        self.__storage_repository.update(storage)

        serialized = self.__serializer.serialize(record)
        doc = {
            "_id": record.id,
            "storage_id": record.storage_id,
            "tags": record.tags,
            "data": (
                Binary(serialized)
                if self.__serializer.binary
                else Binary(serialized.encode())
            ),
        }
        self.__collection.insert_one(doc)

    def update(self, record: IAudioRecord) -> None:
        if not self.__collection.find_one({"_id": record.id}):
            raise ValueError(f"Record with ID {record.id} not found")

        serialized = self.__serializer.serialize(record)
        doc = {
            "storage_id": record.storage_id,
            "tags": record.tags,
            "data": (
                Binary(serialized)
                if self.__serializer.binary
                else Binary(serialized.encode())
            ),
        }
        self.__collection.update_one({"_id": record.id}, {"$set": doc})

    def delete(self, record_id: str) -> None:
        doc = self.__collection.find_one({"_id": record_id})
        if not doc:
            raise ValueError(f"Record with ID {record_id} not found")

        record = self.__serializer.deserialize(
            doc["data"] if self.__serializer.binary else doc["data"].decode()
        )
        storage = self.__storage_repository.get_by_id(record.storage_id)
        if storage:
            storage.remove_audio_record(record_id)
            self.__storage_repository.update(storage)

        self.__collection.delete_one({"_id": record_id})



from bson.binary import Binary
from pymongo import MongoClient
from pymongo.collection import Collection

from ....domain.interfaces import IUserRepository, ISerializer, IUser


class MongoUserRepository(IUserRepository):
    def __init__(
        self,
        serializer: ISerializer,
        mongo_uri: str,
        db_name: str,
        collection_name: str = "users",
    ) -> None:

        self.__serializer = serializer

        self.__client = MongoClient(mongo_uri)
        self.__db = self.__client[db_name]

        self.__collection: Collection = self.__db[collection_name]
        self.__collection.create_index("email", unique=True)

    def get_by_id(self, user_id: str) -> IUser | None:
        doc = self.__collection.find_one({"_id": user_id})
        if doc is None:
            return None

        data = doc["data"] if self.__serializer.binary else doc["data"].decode()
        return self.__serializer.deserialize(data)

    def get_by_email(self, user_id: str) -> IUser | None:
        doc = self.__collection.find_one({"email": user_id})
        if doc is None:
            return None

        data = doc["data"] if self.__serializer.binary else doc["data"].decode()
        return self.__serializer.deserialize(data)

    def add(self, user: IUser) -> None:
        if self.__collection.find_one({"_id": user.id}):
            raise ValueError(f"User with ID {user.id} already exists")
        if self.__collection.find_one({"email": user.email}):
            raise ValueError(f"User with email {user.email} already exists")

        serialized = self.__serializer.serialize(user)
        doc = {
            "_id": user.id,
            "email": user.email,
            "data": (
                Binary(serialized)
                if self.__serializer.binary
                else Binary(serialized.encode())
            ),
        }

        self.__collection.insert_one(doc)

    def update(self, user: IUser) -> None:
        if not self.__collection.find_one({"_id": user.id}):
            raise ValueError(f"User with ID {user.id} not found")

        serialized = self.__serializer.serialize(user)
        doc = {
            "email": user.email,
            "data": (
                Binary(serialized)
                if self.__serializer.binary
                else Binary(serialized.encode())
            ),
        }

        self.__collection.update_one({"_id": user.id}, {"$set": doc})

    def delete(self, user: IUser) -> None:
        if not self.__collection.find_one({"_id": user.id}):
            raise ValueError(f"User with ID {user.id} not found")

        self.__collection.delete_one({"_id": user.id})

    def get_all(self) -> list[IUser]:
        users = self.__collection.find()
        users = [self.get_by_id(user["_id"]) for user in users]

        return users



from datetime import datetime

from .dto.user_dto import UserDTO
from ...domain.entities.user import Admin
from ...domain.factories import AuthUserFactory, AdminFactory, IUserFactory
from ...domain.interfaces import IMapper, IUser


class UserMapper(IMapper):
    def __init__(self):
        self._factories: dict[str, IUserFactory] = {
            "authuser": AuthUserFactory(),
            "admin": AdminFactory(),
        }

    def to_dto(self, user: IUser) -> UserDTO:
        if not isinstance(user, IUser):
            raise TypeError("Object must be a IUser instance")

        entity_type = "admin" if isinstance(user, Admin) else "authuser"

        return UserDTO(
            entity_type=entity_type,
            id=user.id,
            email=user.email,
            password_hash=user.password_hash,
            registration_date=user.registration_date.isoformat(),
            last_updated=user.last_updated.isoformat(),
            is_blocked=user.is_blocked,
            temp_password_hash=user.temp_password_hash,
        )

    def from_dto(self, dto: UserDTO) -> IUser:
        factory = self._factories.get(dto.entity_type)
        if not factory:
            raise ValueError(f"Invalid entity_type: {dto.entity_type}")

        user = factory.create_user(dto.email, dto.password_hash)

        if dto.temp_password_hash:
            user.temp_password_hash = dto.temp_password_hash

        user.is_blocked = dto.is_blocked
        user.id = dto.id
        user.last_updated = datetime.fromisoformat(dto.last_updated)
        user.registration_date = datetime.fromisoformat(dto.registration_date)

        return user



from abc import ABC, abstractmethod
from datetime import datetime


class IAudioRecord(ABC):
    @property
    @abstractmethod
    def id(self) -> str: ...

    @id.setter
    @abstractmethod
    def id(self, id_: str): ...

    @property
    @abstractmethod
    def text(self) -> str: ...

    @text.setter
    @abstractmethod
    def text(self, value: str) -> None: ...

    @property
    @abstractmethod
    def language(self) -> str: ...

    @property
    @abstractmethod
    def tags(self) -> list: ...

    @abstractmethod
    def add_tag(self, tag_name: str, update: bool = True) -> None: ...

    @abstractmethod
    def remove_tag(self, tag_name: str) -> None: ...

    @property
    @abstractmethod
    def record_name(self) -> str: ...

    @record_name.setter
    @abstractmethod
    def record_name(self, note_name: str): ...

    @property
    @abstractmethod
    def storage_id(self) -> str: ...

    @property
    @abstractmethod
    def file_path(self) -> str: ...

    @property
    @abstractmethod
    def last_updated(self) -> datetime: ...

    @last_updated.setter
    @abstractmethod
    def last_updated(self, value: datetime): ...



import hashlib
import secrets

from ...domain.interfaces import IPasswordManager


class PasswordManager(IPasswordManager):
    def hash_password(self, password: str) -> str:
        """
        Hash a password.

        :param password: Password to hash.
        :return: Hash result.
        """

        return hashlib.sha512(password.encode("utf-8")).hexdigest()

    def create_password(self) -> str:
        """
        Create password.

        :return: Not hashed password."""

        password = secrets.token_urlsafe(12)
        return password

    def verify_password(self, hashed_password: str, other: str) -> bool:
        return hashed_password == self.hash_password(other)



from .audio_repository import *
from .local_file_manager import *
from .storage_repository import *
from .user_reposityory import *

__all__ = [
    "LocalAudioRepository",
    "LocalUserRepository",
    "LocalStorageRepository",
    "LocalFileManager",
]



from abc import ABC, abstractmethod


class IMapper(ABC):
    @abstractmethod
    def to_dto(self, data):
        """
        Convert the entity to a DTO.

        :return: DTO representation of the entity.
        """
        pass

    @abstractmethod
    def from_dto(self, dto):
        """
        Create an entity from a DTO.

        :param dto: DTO with entity data.
        :return: Entity instance.
        """
        pass



from abc import ABC, abstractmethod


class ITextExporter(ABC):
    @abstractmethod
    def export_text(
        self, content: str, output_dir: str, filename: str, file_format: str
    ) -> str:
        """
        Export text (string) to format.

        :param content: Target text to export.
        :param output_dir: Target output directory.
        :param filename: Filename to save the exported text.
        :param file_format: Format to save the exported text.
        """
        pass



import smtplib
from email.message import EmailMessage

from ...domain.interfaces import IEmailService


class EmailService(IEmailService):
    def __init__(
        self, smtp_server: str, smtp_port: int, sender_email: str, sender_password: str
    ):
        """
        Initializes the email services for sending recovery emails using an SMTP server.

        :param smtp_server: The SMTP server address (e.g., smtp.gmail.com).
        :param smtp_port: Port number for the SMTP server (commonly 465 for SSL or 587 for TLS).
        :param sender_email: Email address of the sender (used for authentication and sending).
        :param sender_password: Password or app token for the sender's email account.
        """
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port
        self.sender_email = sender_email
        self.sender_password = sender_password

    def send_recovery_email(self, target_email: str, temp_password: str):
        """
        Sends a password recovery email containing the temporary password.

        :param target_email: Recipient's email address.
        :param temp_password: The generated temporary password to include in the email.
        :raises Exception: If there is an error during the sending process.
        """
        message = EmailMessage()
        message["Subject"] = "Password Recovery"
        message["From"] = self.sender_email
        message["To"] = target_email

        message.set_content(
            f"""
        You have requested to recover your account password.
        Your temporary password is: {temp_password}

        Please log in using this password and change it immediately to ensure security.

        Best regards,
        Support Team
        
        Здравствуйте!
        
        Вы отправили запрос на восстановление пароля.
        Ваш временный пароль: {temp_password}.
        
        Войдите, используя этот пароль, и поменяйте его как можно скорее.
        
        Пожалуйста, проигнорируйте данное письмо, если оно попало к Вам по ошибке.
        
        --
        С уважением,
        Служба поддержки
        """
        )

        with smtplib.SMTP(self.smtp_server, self.smtp_port) as server:
            server.ehlo()

            if self.smtp_port == 587:
                server.starttls()

            server.login(self.sender_email, self.sender_password)
            server.send_message(message)



from .istorage_service import IStorageService
from ..serialization import StorageMapper, StorageDTO
from ...domain.factories import IStorageFactory, StorageFactory
from ...domain.interfaces import IStorageRepository


class StorageService(IStorageService):
    """Manages storage creation and retrieval."""

    def __init__(self, storage_repo: IStorageRepository):
        self.__storage_repository = storage_repo
        self.__storage_factory: IStorageFactory = StorageFactory()
        self.mapper = StorageMapper()

    def create_storage(self, user_id: str) -> StorageDTO:
        """Creates a new storage container for a user."""

        storage = self.__storage_factory.create_storage(user_id)
        self.__storage_repository.add(storage)
        return self.mapper.to_dto(storage)

    def get_user_storage(self, user_id: str) -> StorageDTO:
        """Returns storage by user id if it exists else None."""

        storage = self.__storage_repository.get_by_user(user_id)
        return self.mapper.to_dto(storage)

    def remove_audio_record(self, storage_id: str, record_id: str) -> None:
        storage = self.__storage_repository.get_by_id(storage_id)
        storage.remove_audio_record(record_id)
        self.__storage_repository.update(storage)

    def get_all_record_ids(self, storage_id: str) -> list[str] | None:
        storage = self.__storage_repository.get_by_id(storage_id)
        return storage.audio_record_ids if storage else []



from .email_service import *
from .password_manager import *
from .stopwords_remover import *
from .transcriber import *

__all__ = [
    "PasswordManager",
    "Transcriber",
    "StopwordsRemover",
    "EmailService",
]



from .mongo_audio_repository import *
from .mongo_storage_repository import *
from .mongo_user_repository import *

__all__ = ["MongoUserRepository", "MongoAudioRepository", "MongoStorageRepository"]



from setuptools import setup, find_packages

setup(
    name="transcriber_service",
    version="0.1.0",
    description="A toolkit for transcribing",
    author="Tatsiana Kozlova",
    author_email="tanya126060@gmail.com",
    license="MIT",
    packages=find_packages(exclude=["tests*"]),
    install_requires=[
        "pydantic>=2.11.4",
        "typing>=3.7.4.3",
        "password_strength>=0.0.3.post2",
        "email_validator>=2.2.0",
        "python-docx>=1.1.2",
        "pymongo>=4.12.1",
        "msgpack>=1.1.0",
        "audio_transcribing>=0.2.6",
    ],
    python_requires=">=3.8",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.13",
        "Topic :: Software Development :: Libraries",
    ],
    include_package_data=True,
    zip_safe=False,
)



import logging

from email_validator import validate_email
from password_strength import PasswordPolicy

from .user_service import UserService
from ..serialization.dto import UserDTO
from ..serialization.user_mapper import UserMapper
from ...domain import AuthException
from ...domain.factories import (
    AuthUserFactory,
    IUserFactory,
    AdminFactory,
)
from ...domain.interfaces import IEmailService, IPasswordManager

logger = logging.getLogger(__name__)


class AuthService(object):
    """
    Represents an authentication services.
    Supports login, register, changing and recovering password, blocking and unblocking of users.
    """

    def __init__(
        self,
        user_service: UserService,
        email_service: IEmailService,
        password_hasher: IPasswordManager,
    ):
        self._user_service = user_service
        self._password_manager = password_hasher
        self._policy = PasswordPolicy.from_names(
            length=8, uppercase=1, numbers=1, special=1
        )
        self.mapper = UserMapper()
        self.__email_service = email_service

    def _register(self, email: str, password: str, factory: IUserFactory) -> UserDTO:
        if self._user_service.get_user_by_email(email):
            logger.error(f"User with email {email} already exists")
            raise ValueError("User already exists")

        errors = self._policy.test(password)
        if errors:
            logger.error(f"Error validating password: {errors}, {password}")
            raise AuthException(
                f"Password is weak: 8 symbols, 1 uppercase, number, special"
            )

        email = validate_email(email).normalized
        password_hash = self._password_manager.hash_password(password)
        user = self._user_service.create_user(email, password_hash, factory)

        return self.mapper.to_dto(user)

    def register_user(self, email: str, password: str) -> UserDTO:

        return self._register(email, password, AuthUserFactory())

    def create_admin(self, email: str, password: str) -> UserDTO:
        return self._register(email, password, AdminFactory())

    def login(self, email: str, password: str) -> UserDTO:

        user = self._user_service.get_user_by_email(email)
        if not user or not password:
            logger.warning(f"User {email} not found or no password.")
            raise AuthException("Invalid credentials")
        if user.temp_password_hash and self._password_manager.verify_password(
            user.temp_password_hash, password
        ):
            logger.info(f"User {email} use temp password.")
            user.password_hash = user.temp_password_hash
            user.temp_password_hash = None
            self._user_service.update_user(user)
        if not self._password_manager.verify_password(user.password_hash, password):
            logger.warning(f"User {email} login unsuccessful.")
            raise AuthException("Invalid credentials")

        if user.is_blocked:
            raise AuthException("User is blocked")

        return self.mapper.to_dto(user)

    def change_password(
        self, email: str, current_password: str, new_password: str
    ) -> None:

        user = self._user_service.get_user_by_email(email)
        if not user:
            raise AuthException("User not found")

        errors = self._policy.test(new_password)
        if errors:
            raise AuthException(f"Password is weak: {errors}")
        if not self._password_manager.verify_password(
            user.password_hash, current_password
        ):
            raise AuthException("Incorrect password")

        user.password_hash = self._password_manager.hash_password(new_password)
        self._user_service.update_user(user)

    def recover_password(self, email: str) -> None:
        user = self._user_service.get_user_by_email(email)
        if not user:
            raise AuthException("User not found")

        temp_password = self._password_manager.create_password()
        temp_password_hash = self._password_manager.hash_password(temp_password)
        user.temp_password_hash = temp_password_hash

        self.__email_service.send_recovery_email(user.email, temp_password)
        self._user_service.update_user(user)



from typing import Tuple

from ..interfaces import IAudioRecord


class AudioSearchService(object):
    @staticmethod
    def search_by_tags(
        records: Tuple[IAudioRecord, ...], tags: list[str], match_all: bool = False
    ) -> Tuple[IAudioRecord, ...]:
        """
        Search audio records by tags.

        :param records: Audio records to search.
        :param tags: List of tags to search for.
        :param match_all: True if record must contain all tags.
        :return: Tuple of matching audio records.
        :raises ValueError: If tags is empty or invalid.
        """
        tags = [tag.lower() for tag in tags]

        def matches(record: IAudioRecord) -> bool:
            if match_all:
                return all(tag in record.tags for tag in tags)
            return any(tag in record.tags for tag in tags)

        return tuple(record for record in records if matches(record))

    @staticmethod
    def search_by_name(
        records: Tuple[IAudioRecord, ...], name: str
    ) -> Tuple[IAudioRecord, ...]:
        """
        Search audio records by name.

        :param records: Audio records to search.
        :param name: Record name to match.
        :return: Tuple of matching audio records.
        :raises ValueError: If name is invalid.
        """
        name = name.lower()

        return tuple(record for record in records if name in record.record_name.lower())



from typing import Any

from pydantic import BaseModel

from .dto import UserDTO, StorageDTO, AudioRecordDTO
from .entity_mapper_factory import EntityMapperFactory
from ...domain.interfaces import ISerializer


class SerializerAdapter(ISerializer):
    def __init__(
        self,
        base_serializer: ISerializer,
        entity_mapper_factory: EntityMapperFactory,
    ):
        self._base_serializer = base_serializer
        self._entity_mapper_factory = entity_mapper_factory
        self._dto_mapping = {
            "authuser": UserDTO,
            "admin": UserDTO,
            "storage": StorageDTO,
            "audiorecord": AudioRecordDTO,
        }

    def serialize(self, obj) -> str | bytes:
        """
        Serialize data, converting entities to DTOs.

        :param obj: Data to serialize (entity or DTO).
        :return: Serialized data (string or bytes).
        """

        data = self._to_dto(obj)
        return self._base_serializer.serialize(data)

    def deserialize(self, data: str | bytes):
        """
        Deserialize data, restoring entities from DTOs.

        :param data: Serialized data.
        :return: Deserialized data (entities or DTOs).
        """

        deserialized = self._base_serializer.deserialize(data)
        return self._from_dto(deserialized)

    def _to_dto(self, obj):
        """Convert an entity to a DTO if applicable."""

        if isinstance(obj, BaseModel):
            return obj.model_dump()
        if isinstance(obj, (list, tuple, set)):
            return [self._to_dto(item) for item in obj]
        elif isinstance(obj, dict):
            return {key: self._to_dto(value) for key, value in obj.items()}
        elif (
            hasattr(obj, "__class__")
            and obj.__class__.__name__.lower() in self._dto_mapping
        ):
            serializer = self._entity_mapper_factory.get_mapper(
                obj.__class__.__name__.lower()
            )
            return serializer.to_dto(obj).model_dump()
        return obj

    def _from_dto(self, data: Any) -> Any:
        """Restore an entity from a DTO if applicable."""
        if isinstance(data, (list, tuple)):
            return [self._from_dto(item) for item in data]
        elif isinstance(data, dict) and "entity_type" in data:
            dto_class = self._dto_mapping.get(data["entity_type"])
            if dto_class:
                dto = dto_class(**data)
                serializer = self._entity_mapper_factory.get_mapper(data["entity_type"])
                return serializer.from_dto(dto)
        elif isinstance(data, dict):
            return {key: self._from_dto(value) for key, value in data.items()}
        return data

    @property
    def extension(self) -> str:
        return self._base_serializer.extension

    @property
    def binary(self) -> bool:
        return self._base_serializer.binary



from datetime import datetime
from uuid import uuid4

from ..exceptions import AuthException
from ..interfaces import IUser


class User(IUser):
    """
    Represents a base user in the system.

    Handles core user properties validation and initialization.
    Should not be instantiated directly - use AuthUser or Admin subclasses.
    """

    def __init__(self, email: str, password_hash: str):
        """
        Create User instance with validated credentials.

        :param email: User's email address.
        :param password_hash: Plain-text password to be hashed.
        :raises: If email is not valid.
        """

        self._email: str = email
        self._id: str = uuid4().hex
        self._password_hash: str = password_hash
        self._temp_password_hash: str | None = None
        self._registration_date: datetime = datetime.now()
        self._last_updated: datetime = self._registration_date
        self._is_blocked: bool = False

    @property
    def is_blocked(self) -> bool:
        """Returns True if the user is blocked else False."""

        return self._is_blocked

    @is_blocked.setter
    def is_blocked(self, is_blocked: bool):
        """Sets the blocked state of the user."""

        self._is_blocked = is_blocked
        self._last_updated = datetime.now()

    @property
    def password_hash(self) -> str:
        return self._password_hash

    @password_hash.setter
    def password_hash(self, password_hash: str):
        if not password_hash:
            raise AuthException("Password hash cannot be empty.")
        self._password_hash = password_hash
        self._last_updated = datetime.now()

    @property
    def temp_password_hash(self) -> str | None:
        return self._temp_password_hash

    @temp_password_hash.setter
    def temp_password_hash(self, temp_password_hash: str):
        self._temp_password_hash = temp_password_hash
        self._last_updated = datetime.now()

    @property
    def id(self) -> str:
        """Returns user ID."""

        return self._id

    @id.setter
    def id(self, id_: str):
        """Sets the user ID."""
        self._id = id_

    @property
    def email(self) -> str:
        """Returns user email."""

        return self._email

    @property
    def registration_date(self) -> datetime:
        """Returns user registration date."""

        return self._registration_date

    @property
    def last_updated(self) -> datetime:
        """Returns user last updated date."""

        return self._last_updated

    @last_updated.setter
    def last_updated(self, value: datetime):
        """Sets the user last updated date."""
        self._last_updated = value

    def can_block(self) -> bool:
        raise NotImplementedError()

    @registration_date.setter
    def registration_date(self, value: datetime):
        """Sets the user registration date."""
        self._registration_date = value


class AuthUser(User):
    """
    Represents an authenticated user with basic privileges.

    Inherits from User class and sets default role to 'user'.
    Should be used for all registered non-admin users.
    """

    def __init__(self, email: str, password_hash: str):
        super().__init__(email, password_hash)

    def can_block(self) -> bool:
        return False


class Admin(AuthUser):
    """
    Represents an administrator user with elevated privileges.

    Inherits from AuthUser and sets default role to 'admin'.
    Should be instantiated only through proper admin creation process.
    """

    def __init__(self, email: str, password_hash: str):
        super().__init__(email, password_hash)

    def can_block(self) -> bool:
        return True



from ....domain.interfaces import (
    IAudioRepository,
    IStorageRepository,
    IAudioRecord,
    IFileManager,
    ISerializer,
)


class LocalAudioRepository(IAudioRepository):
    def __init__(
        self,
        storage_repository: IStorageRepository,
        data_dir: str,
        file_manager: IFileManager,
        serializer: ISerializer,
    ) -> None:
        """
        Create local audio repository.

        :type storage_repository: IStorageRepository.
        :param data_dir: Directory to store local audio data.
        """
        self.__serializer = serializer
        self.__file_manager = file_manager
        self.__records: dict[str, IAudioRecord] = {}
        self.__storage_repository = storage_repository
        self.__dir = data_dir

        try:
            self.__records = file_manager.load(self.__dir, serializer)
        except:
            self.__records = {}

    def get_by_storage(self, storage_id: str) -> tuple[IAudioRecord, ...]:
        """Return list of audio records by storage id."""
        return tuple(r for r in self.__records.values() if r.storage_id == storage_id)

    def get_by_id(self, record_id: str) -> IAudioRecord | None:
        """Return audio record by id if it exists else None."""

        return self.__records.get(record_id)

    def add(self, record: IAudioRecord) -> None:
        """
        Add audio record to repository.

        :raise ValueError: If audio record already exists.
        """

        if record.id in self.__records:
            raise ValueError("Record already exists.")

        storage = self.__storage_repository.get_by_id(record.storage_id)
        if storage:
            storage.add_audio_record(record.id)
            self.__storage_repository.update(storage)
            self.__records[record.id] = record
            self.__save()

    def update(self, record: IAudioRecord) -> None:
        """
        Update audio record from repository with new value.

        :param record: new value of audio record.
        :raise ValueError: If audio record does not exist.
        """

        if record.id not in self.__records:
            raise ValueError("Record not found.")
        self.__records[record.id] = record
        self.__save()

    def delete(self, record_id: str) -> None:
        """
        Delete audio record from repository.

        :param record_id: ID of target audio record.
        :raise ValueError: If audio record does not exist.
        """

        record = self.__records.get(record_id)
        if not record:
            raise ValueError("Record not found.")

        storage = self.__storage_repository.get_by_id(record.storage_id)
        if storage:
            storage.remove_audio_record(record_id)
            self.__storage_repository.update(storage)

        del self.__records[record_id]
        self.__save()

    def __save(self) -> None:
        self.__file_manager.save(self.__records, self.__dir, self.__serializer)



from datetime import datetime

from .dto.audio_record_dto import AudioRecordDTO
from ...domain.factories import AudioRecordFactory, IAudioRecordFactory
from ...domain.interfaces import IMapper, IAudioRecord


class AudioRecordMapper(IMapper):
    def __init__(self, factory: IAudioRecordFactory = AudioRecordFactory()):
        self._factory = factory

    def to_dto(self, audio: IAudioRecord) -> AudioRecordDTO:
        if not isinstance(audio, IAudioRecord):
            raise TypeError("Object must be an IAudioRecord instance")

        return AudioRecordDTO(
            entity_type="audiorecord",
            id=audio.id,
            record_name=audio.record_name,
            file_path=audio.file_path,
            storage_id=audio.storage_id,
            text=audio.text,
            language=audio.language,
            tags=audio.tags,
            last_updated=audio.last_updated.isoformat(),
        )

    def from_dto(self, dto: AudioRecordDTO) -> IAudioRecord:
        audio = self._factory.create_audio(
            file_name=dto.record_name,
            file_path=dto.file_path,
            storage_id=dto.storage_id,
            text=dto.text or "",
            language=dto.language or "",
        )
        audio.id = dto.id
        audio.last_updated = datetime.fromisoformat(dto.last_updated)
        for tag in dto.tags:
            audio.add_tag(tag, False)

        return audio



import logging

from ..serialization.audio_mapper import (
    AudioRecordDTO,
    AudioRecordMapper,
)
from ...domain.factories import IAudioRecordFactory, AudioRecordFactory
from ...domain.interfaces import (
    IAudioRepository,
    ITextExporter,
    IStopwordsRemover,
    ITranscriber,
)
from ...domain.services.audio_search_service import AudioSearchService

logger = logging.getLogger(__name__)


class AudioRecordService(object):
    """
    Service class for managing audio records and their lifecycle operations.

    Provides functionality to create and retrieve audio records with optional
    transcription processing.
    """

    def __init__(
        self,
        repo: IAudioRepository,
        transcriber: ITranscriber,
    ):
        self._MAX_SIZE = 1024 * 1024 * 10
        self._repository = repo
        self._transcriber = transcriber
        self._audio_factory: IAudioRecordFactory = AudioRecordFactory()
        self._search_service = AudioSearchService()
        self.mapper = AudioRecordMapper()

    def create_audio(
        self,
        file_name: str,
        content: bytes,
        file_path: str,
        storage_id: str,
        language: str = None,
        max_speakers: int = None,
        main_theme: str = None,
    ) -> AudioRecordDTO:
        """
        Create AudioRecord instance with basic metadata and do transcription
        into text with given transcribe services.

        :param file_name: Name of audio file.
        :param content: Content of audio file (mp3).
        :param file_path: Full path to audio file in some storage directory
        (not user storage).
        :param storage_id: Storage id of audio file.
        :param language: Language of audio file (defaults None).
        :param max_speakers: Max number of speakers (defaults None).
        :param main_theme: Main theme of audio file (defaults None).
        :return: Created Audio Record.
        """

        if len(content) > self._MAX_SIZE:
            raise Exception("Audio file too large.")
        text, language = self._transcriber.transcribe(
            content, language, max_speakers, main_theme
        )
        audio = self._audio_factory.create_audio(
            file_name, file_path, storage_id, text, language
        )
        self._repository.add(audio)
        return self.mapper.to_dto(audio)

    def get_records(self, storage_id: str) -> tuple[AudioRecordDTO, ...]:
        """
        Retrieves audio record by its storage container ID.

        :param storage_id: Storage id of audio file.
        :return: Tuple of audio records if it is found else None.
        """
        records = self._repository.get_by_storage(storage_id)
        return tuple(self.mapper.to_dto(record) for record in records)

    def get_by_id(self, record_id: str) -> AudioRecordDTO:
        record = self._repository.get_by_id(record_id)
        return self.mapper.to_dto(record)

    def search_by_tags(
        self, storage_id: str, tags: list[str], match_all: bool = False
    ) -> list[AudioRecordDTO]:

        records = self._repository.get_by_storage(storage_id)
        matching_records = self._search_service.search_by_tags(records, tags, match_all)
        return [self.mapper.to_dto(record) for record in matching_records]

    def search_by_name(self, storage_id: str, name: str) -> list[AudioRecordDTO]:

        records = self._repository.get_by_storage(storage_id)
        matching_records = self._search_service.search_by_name(records, name)
        return [self.mapper.to_dto(record) for record in matching_records]

    def delete(self, record_id):
        self._repository.delete(record_id)


class AudioTagService(object):
    def __init__(self, repository: IAudioRepository):
        self._repository = repository

    def add_tag_to_record(self, record_id: str, tag: str) -> None:
        record = self._repository.get_by_id(record_id)
        if not record:
            raise ValueError("Record not found")

        record.add_tag(tag)
        self._repository.update(record)

    def remove_tag_from_record(self, record_id: str, tag: str) -> None:
        record = self._repository.get_by_id(record_id)
        if not record:
            raise ValueError("Record not found")

        try:
            record.remove_tag(tag)
            self._repository.update(record)
        except ValueError:
            pass


class AudioTextService(object):
    def __init__(
        self,
        repository: IAudioRepository,
        export_service: ITextExporter,
        stopwords_remover: IStopwordsRemover,
    ):
        self._repository = repository
        self._export_service = export_service
        self._stopwords_remover = stopwords_remover

    def export_record_text(
        self, record_id: str, output_dir: str, file_format: str
    ) -> str:
        record = self._repository.get_by_id(record_id)
        if not record:
            raise ValueError("Audio record not found")

        return self._export_service.export_text(
            record.text, output_dir, record.record_name, file_format
        )

    def remove_stopwords(
        self,
        record_id: str,
        remove_swear_words: bool = True,
        go_few_times: bool = False,
    ) -> None:
        record = self._repository.get_by_id(record_id)
        if not record:
            logger.warning("Audio record not found")
            raise ValueError("Audio record not found")
        if record.language.lower() not in ["ru", "russian"]:
            logger.warning(f"Unsupported language: {record.language}")
            raise ValueError(f"Unsupported language: {record.language}")
        logger.info(f"Removing stopwords for text: {record.text}")

        updated_text = self._stopwords_remover.remove_stopwords(
            record.text, remove_swear_words, go_few_times
        )

        logger.info(f"Text after removing stopwords: {updated_text}")
        record.text = updated_text
        self._repository.update(record)

    def remove_words(self, record_id: str, words: list | tuple) -> None:
        record = self._repository.get_by_id(record_id)
        if not record:
            raise ValueError("Audio record not found")
        if record.language.lower() not in ("ru", "russian"):
            raise ValueError(f"Unsupported language: {record.language}")

        record.text = self._stopwords_remover.remove_words(record.text, words)
        self._repository.update(record)

    def change_record_name(self, record_id: str, name: str) -> None:
        record = self._repository.get_by_id(record_id)
        if not record:
            raise ValueError("Record not found")

        record.record_name = name
        self._repository.update(record)
